import platform
import datetime
import getpass
from termcolor import cprint
from .logger import Logger, LogColors

class Info():
    def __init__(self) -> None:
        # self.banner = cprint(figlet_format(
        #     'VPNroulette', font='smslant'), 'green')
        self.log = Logger()

    # def system_info(self) -> None:
    #     self.log.info(f"OS: {platform.system()}")
    #     self.log.info(f"SYSTEM USER: {getpass.getuser()}")
    #     self.log.info(f"Time: {datetime.datetime.now()}")

    def banner(self) -> None:
        """
            Print VPNR banner in the command line
        """
        cprint(f"""
                ,,,,,,,                
            ,,,,,,,,,,,,,,,            
        ,,,,,,,///////////,,,,,,,       
    ,,,,,,,//////,,,,/////,,,,,,,,,,,   
    ,,,,,/////,,///////////////,,,,,,   
    ,,,//////,/////,,,,,,,///////,,,,       Status: test
    ,,,/////,////,////////,,//////,,,       Country: test
    ,,/////,,///,,/////,///,,/////,,,       Test: test
    ,,,/////,*///,,//,,,///,,/////,,,       OS: {platform.system()}
    ,,,//////,,//////////,,//////,,,,       USER: {getpass.getuser()}
    ,,,,////////,,,,,,,,,///////,,,,,       Time: {datetime.datetime.now()}
    ,,,,,,////////////////////,,,,,,,   
        .,,,,,////////////,,,,,,        
            ,,,,,,,,,,,,,,,            
                ,,,,,,,
        """, "green")

    def disclaimer(self):
        """
            Print VPNR disclaimer
        """
        cprint(
        f"""
        {LogColors.ERROR}-----------------------------------------< D I S C L A I M E R >------------------------------------------{LogColors.NOCOLOR}
        {LogColors.ERROR}VPNroulette is a free VPN service that allows you to connect to any country in the world.{LogColors.NOCOLOR}
        {LogColors.ERROR}This software comes without any kind of guarantee and it's going to be used under your own responsability.{LogColors.NOCOLOR}
        {LogColors.ERROR}Please don't be evil, or your account will be suspended inmediately without notice{LogColors.NOCOLOR}
        {LogColors.ERROR}For more information visit: https://vpnroulette.com{LogColors.NOCOLOR}
        """)



import argparse


def parse_args():
    parser = argparse.ArgumentParser(
        description="""VPNROULETTE CLIENT""",
        add_help=True,
        prog="vpnroulette"
    )
    parser.add_argument(
        "-l",
        "--level",
        choices=["INFO", "WARNING", "ERROR", "CRITICAL", "DEBUG"],
        required=False,
        dest="log_level",
        default="DEBUG",
        help="""level of logging""",
        type=str,
    )
    parser.add_argument(
        "-d",
        "--disclaimer",
        required=False,
        dest="disclaimer",
        action='store_true',
        help="""Display VPNR disclaimer""",
    )
    parser.add_argument('status', nargs='?')
    # parser.add_argument(
    #     "connect",
    #     nargs='?',
    #     required=False,
    #     dest="connect",
    #     # action="store_true",
    #     help="""Connect vpnroulette in our servers""",
    # )
    # parser.add_argument(
    #     "-c",
    #     "--clean",
    #     required=False,
    #     dest="clean",
    #     action='store_true',
    #     help="""Clean credentials""",
    # )

    return parser.parse_args()



import os
import platform
import getpass
import json
import requests
from itsdangerous import base64_decode
from .logger import Logger
from .utils import Utils


class Configuration():
    """
        Configuration of the app
    """
    def __init__(self) -> None:
        self.log = Logger()
        self.utils = Utils()
        self.vpnr_dir = os.path.join(os.environ.get('HOME'), '.vpnr-client')
        self.config_file = os.path.join(self.vpnr_dir, 'config')
        self.token_file = os.path.join(self.vpnr_dir, 'token')
        self.tunnel_ovpn_file = os.path.join(self.vpnr_dir, 'tunnel.ovpn')

    def get_token(self, email: str, password: str) -> None:
        """
            Get token with your login credentials to get the tunnel
        """

        data = {'email': email, 'password': password}
        url = "http://api2.vpnroulette.net:8181/auth/login"
        r = requests.post(url, data=data)

        self.log.info("Trying to get token")
        if r.status_code == 200:
            self.log.info("You are successfully loged in!")
            self.utils.write_config(r.json(),self.token_file)
        else:
            self.log.error_and_exit(f"{r.status_code}: Bad credentials, can't get the token")

    def credentials(self) -> str:
        """
            Ask credentials with your VPNR account
        """
        email = getpass.getpass("Introduce your email: ")
        password = getpass.getpass("Introduce your password: ")
        data = {
            "email": email,
            "password": password
        }
        self.utils.write_config(data,self.config_file)
        self.get_token(email,password)

    def config_exists(self) -> None:
        """
            Check if config exists
        """
        if not os.path.isfile(self.config_file):
            self.log.warn(f"Config file {self.config_file} don't exists, creating new one")
            if not os.path.isdir(self.vpnr_dir):
                os.mkdir(self.vpnr_dir)
            self.credentials()
        else:
            config = self.utils.read_config(self.config_file)
            self.get_token(config["email"],config['password'])

    def get_tunnel_config(self) -> None:
        """
            Get your openvpn config using your credentials
        """
        token = self.utils.read_config(self.token_file)["auth_token"]
        headers = {
            'Authorization': 'Token: ' + token,
            'Content-Type': 'application/json',
            'User-Agent': platform.system(),  # Darwin, Linux, Windows
        }
        r = requests.get('http://api2.vpnroulette.net:8181/tunnel/random', headers=headers)

        self.log.info("Trying to get token")
        if r.status_code == 200:
            self.log.info("Status ok, saving tunnel.ovpn")
            tunnel_encoded_data = json.loads(r.text)
            tunnel_data = base64_decode(tunnel_encoded_data['content'])
            tunnel = str(tunnel_data, 'utf-8')
            print(self.tunnel_ovpn_file)
            self.utils.write_tunnel_config(tunnel,self.tunnel_ovpn_file)
             
            if platform.system() == "Darwin":
                self.log.info("Tunnel data specific for Mac OSX")
                with open(self.tunnel_ovpn_file, 'a+') as file:
                    file.write('script-security 2\n')
                    file.write('up /usr/local/bin/update-resolv-conf\n')
                    file.write('down /usr/local/bin/update-resolv-conf\n')
                    file.write('dhcp-option DNS 8.8.8.8\n')
                    file.write('\n')

    def clean_config(self):
        """
            Clean VPNR configurations
        """
        validation = input(str(
            "This will remove your current config and token file, are you sure [yes/no]? "))
        if validation == "yes":
            self.log.warn(f"Removing {self.config_file}")
            os.remove(self.config_file)
            self.log.warn(f"Removing {self.token_file}")
            os.remove(self.token_file)
        else:
            self.log.warn("Nothing to do with -c clean flag")



import subprocess
from .logger import Logger
import json


class Utils:
    def __init__(self) -> None:
        self.log = Logger()

    def write_config(self, config, config_path) -> None:
        """
            Store config file inside ~/.vpnr-client/config
        """
        json_object = json.dumps(config, indent=4)

        with open(config_path, "w") as file:
            self.log.info(f"Storing json credentials inside {config_path}")
            file.writelines(json_object)
            subprocess.call(['chmod', '0700', config_path])

    def read_config(self, config_path):
        """
            Read file and return json object
        """
        with open(config_path, "r") as file:
            self.log.info(f"Loading credentials from {config_path}")
            credentials = json.load(file)
            return credentials
    
    def write_tunnel_config(self, config, config_path) -> None:
        """
            Write tunnel config
        """
        with open(config_path, "w") as file:
            self.log.info(f"Storing json credentials inside {config_path}")
            file.write(config)
            subprocess.call(['chmod', '0700', config_path])



"""Setup vpnroulette client"""

# Standard library imports
import pathlib
from setuptools import setup, find_packages
HERE = pathlib.Path(__file__).resolve().parent
README = (HERE / "README.md").read_text()

# This call to setup() does all the work
setup(
    name='vpnroulette',
    version='0.0.1',
    description='VPNROULETTE CLIENT',
    long_description=README,
    long_description_content_type="text/markdown",
    python_requires=">=3.6, <4",
    url='https://github.com/vpnroulette/vpnr-client',
    author='Vpnroulette',
    author_email='hello@vpnroulette.com',
    license='Apache',
    install_requires=['requests==2.27.1',
                      'pyfiglet', 'termcolor','itsdangerous'],
    packages=find_packages(include=['vpnr_client']),
    include_package_data=True,
    entry_points={'console_scripts': ['vpnroulette=vpnr_client.__main__:main']},
    classifiers=[
        'Development Status :: 4 - Beta',
        'Intended Audience :: End Users/Desktop',
        'License :: OSI Approved :: Apache Software License',
        'Operating System :: POSIX :: Linux',
        'Operating System :: MacOS',
        "Programming Language :: Python",
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: 3.7',
        'Programming Language :: Python :: 3.8',
        'Programming Language :: Python :: 3.9',
        'Programming Language :: Python :: 3.10',
    ],
)



import os
import subprocess
from .logger import Logger
from .config import Configuration

class Tunnel:
    """
        Connection with our VPNR servers
    """
    def __init__(self) -> None:
        self.log = Logger()
        self.tunnel_config = os.path.expanduser('~/.vpnr-client/tunnel.ovpn')
        self.configuration = Configuration()

    def start_tunnel(self):
        """
            Start the tunnel
        """

        if os.geteuid() != 0:
            self.log.info("You are not root, we need sudo permissions: ")
            subprocess.Popen(['sudo', 'openvpn', '--config', self.tunnel_config])
        else:
            subprocess.Popen(['sudo', 'openvpn', '--config', self.tunnel_config])





"""Vpnroulette command line.
Usage:
------
    $ realpython [options] [id] [id ...]

Available options are:
    -h, --help         Show this help
    -l, --show-links   Show links in text
Contact:
--------
- https://realpython.com/contact/
More information is available at:
- https://pypi.org/project/realpython-reader/
- https://github.com/realpython/reader
Version:
--------
- realpython-reader v1.0.0
"""

from .config import Configuration
from .info import Info
from .tunnel import Tunnel
from .parser import parse_args

def main() -> None:
    """
        Start VPNROULETTE client
    """
    args = parse_args()
    connection = Tunnel()

    info = Info()
    info.banner()

    if args.disclaimer:
        info.disclaimer()

    if args.status:
        print(args.status)
        #connection.start_tunnel()

    config = Configuration()
    config.config_exists()
    config.get_tunnel_config()
    tunnel = Tunnel()
    tunnel.start_tunnel()

    # # Implementing -c flag to allow credentials deletion and try to create a new ones again
    # # if args.clean:
    # #     config.clean_config()


if __name__ == "__main__":
    main()



import logging

class LogColors:
    BLUE1 = '\033[95m'
    BLUE2 = '\033[96m'
    SUCCESS = '\033[92m'
    WARNING = '\033[93m'
    ERROR = '\033[91m'
    NOCOLOR = '\033[0m'
    BOLD = '\033[1m'
    UNDERLINE = '\033[4m'
    START_ERROR = ERROR + 'ERROR |❌| '
    START_WARNING = WARNING + '|🚀| '
    START_INFO = BLUE2 + '|🚀| '
    START_TITLE = WARNING + '  |·'
    START_SUBTITLE = BLUE1 + '  |·'
    START_SUBTITLE_2 = BLUE2 + '  |·'
    START_DEBUG = BLUE2 + '|DEBUG| '
    START_SUCCESS = SUCCESS + '|🚀| '
    LOADING = BLUE1 + 'ººº' + NOCOLOR
    INTERSECTION = ERROR + ' · '


class Logger:
    def __init__(self):
        """
            Main logger class for the command line
        """
        self.log= logging.basicConfig(
            format="%(levelname)-2s %(message)s",
            level="INFO",
            # filename='/tmp/python-telegram-bot.log',
            encoding='utf-8'
        )

    def error(self, msg: str, exception: Exception = None):
        """
            Returns error log with exit 1
        """
        logging.error(f"{LogColors.START_ERROR}{msg}{LogColors.NOCOLOR}")
        exit(1)

    def warn(self, msg: str, exception: Exception = None):
        """
            Return warning message
        """
        logging.warning(f"{LogColors.START_WARNING}{msg}{LogColors.NOCOLOR}")

    def info(self, msg: str, exception: Exception = None):
        """
            Return info message
        """
        logging.info(f"{LogColors.START_SUCCESS}{msg}{LogColors.NOCOLOR}")

    def debug(self, msg: str, exception: Exception = None):
        """
            Return debug message
        """
        logging.debuf(f"{LogColors.START_DEBUG}{msg}{LogColors.NOCOLOR}")
